<?php
namespace Teknisa\OPL\Modules\Consolidacao\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\Libs\Util\DateUtil;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\OPL\Util\Attachment;
use Teknisa\OPL\Util\OplUtilities;
use Teknisa\OPL\Modules\Consolidacao\Services\OPL00800ConclusaoAutomaticaMigracaoService;

/***************/
use Teknisa\Libs\Util\SMTP;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class OPL00800ConclusaoAutomaticaMigracaoController extends \Zeedhi\Framework\Controller\Report {
    protected $entityManager;
    protected $birtStrategy;
    protected $newCode;
    protected $environment;
    protected $connection;
    protected $dataSourceManager;
    protected $qr2Strategy;
    protected $logEmail;
    protected $cookieController;
    protected $oplUtilities;
    protected $globalReports = [];
    protected $smtpUtil;

    public function __construct($reportService, $entityManager, $reportStrategies, $newCode, $environment, $connection, $dataSourceManager, $qr2Strategy, $cookieController, OplUtilities $oplUtilities, OPL00800ConclusaoAutomaticaMigracaoService $service, $smtpUtil) {
        $this->entityManager     = $entityManager;
        $this->reportStrategies  = $reportStrategies;
        $this->reportService     = $reportService;
        $this->newCode           = $newCode;
        $this->environment       = $environment;
        $this->connection        = $connection;
        $this->dataSourceManager = $dataSourceManager;
        $this->qr2Strategy       = $qr2Strategy;
        $this->cookieUtil        = $cookieController;
        $this->oplUtilities      = $oplUtilities;
		$this->service           = $service;
        $this->logEmail          = array('REPORT' => array(), 'SUCCESS' => array(), 'ERROR' => array());
        $this->smtpUtil          = $smtpUtil;
    }

    public function conclusaoAutomaticaMigracao(DTO\Request $request, DTO\Response $response) {
        $isEmailSend = false;
        $stFilialOL = self::sendMailFinalFornecedor();
        
        if(!empty($stFilialOL)){ 
            try{
                $isEmailSend = true;
                self::sendMailFinalGestor($stFilialOL);    
            } catch (\Exception $e) {
                $isEmailSend = false;
            }
            if($isEmailSend) {
                $this->logEmail['SUCCESS'][] = array("MESSAGE" => "EMAIL_SEND_SUCCESS");
            } else {
                $this->logEmail['ERROR'][] = array("MESSAGE" => "EMAIL_SEND_FAILED");
            }
            $response->addMessage(new Message($this->logEmail, 'W'));
        }
        if($isEmailSend){
            $todas    = "W";
            $rupturas = $this->service->getRuptura($todas); 
            if(!empty($rupturas[0])) {
                foreach($rupturas as $ruptura) {
                    $this->service->atualizaLogrupturaolautFinal($ruptura["NRLOGRUPTURAOLAUT"]);                     
                }        
            }
        } 
    }
    
    
    public function sendMailFinalFornecedor() {
        $localResponse = new DTO\Response();
        $todas    = "T";
        $rupturas = $this->service->getRuptura($todas); 
        if(!empty($rupturas[0])) {
            $rupturas = ($rupturas[0]);
            $rupturas = $rupturas["CDFILIAL"];
            $filialOL = $rupturas;
        }    
        $processo = 'EMAIL';
        $todas    = "N";
        $rupturas = $this->service->getRuptura($todas); 
        $chave    = '_';
        if(!empty($rupturas[0])) {
            foreach($rupturas as $ruptura) {
                if($chave<>($ruptura["LISTCDOPERADOR"] . $ruptura["LISTCDFILDEST"] . $ruptura["CDFILIAL"] . $ruptura["NRSEQRUPTURAOL"] . $ruptura["CDFORNECED"])){
                    $chave = $ruptura["LISTCDOPERADOR"] . $ruptura["LISTCDFILDEST"] . $ruptura["CDFILIAL"] . $ruptura["NRSEQRUPTURAOL"] . $ruptura["CDFORNECED"];
                    $rows    = $this->service->getParasupri($ruptura["CDFILIAL"]);
                    foreach($rows as $row1s) {
                        if(!empty($row1s["NMFILIAL"])) {
                            if(!empty($row1s["DSEMAILCONCLAUTRUPOL"])){
                                if((!empty($row1s["DSSMTPEMAILF"])) && (!empty($row1s["DSLOGINSMTP"])) && (!empty($row1s["DSEMAILSMTP"]))) {
                                    $fornecedores = $this->service->getFornecedorRup($ruptura);
                                    if(!empty($fornecedores[0])) {
                                        foreach($fornecedores as $fornecedor) {
                                            if ($fornecedor["IDTPENVAFNORMAL"] == "P"){
                                                $dadosForns = $this->service->getDadosForn($fornecedor["CDFORNECEDENVIO"]);
                                                if(!empty($dadosForns[0])){
                                                    foreach($dadosForns as $dadosForn) { 
                                                        if(!empty($dadosForn["DSEMAILCOT"])){
                                                            $result           = self::tratarMensagemCorpoEmailForn($dadosForn["CDFORNECED"], $dadosForn["NRINSJURFORN"], 
                                                                                                                    $dadosForn["NMFANTFORN"], $fornecedor["NRSEQENVIOAFORN"], $fornecedor["NRAUTOFORN"]);
                                                            $message          = $result[0];
                                                            $attachments      = $result[1];
                                                            $clearAttachments = $result[2];
                                                            $assunto          = "Autorização de Fornecimento";
                                                            $messageSendMail  = $this->oplUtilities->messageSendMail($filialOL, $assunto, $message, 'TECFOOD'); 
                                                            try {
                                                                try{
                                                                    $isEmailSend = true;
                                                                    self::sendMailFinal($row1s["CDSENHEMAILF"], $row1s["DSSMTPEMAILF"], $row1s["DSLOGINSMTP"], $row1s["DSEMAILSMTP"], $row1s["TIPOCRIPTO"], $dadosForn["DSEMAILCOT"], $messageSendMail, $assunto, $dadosForn["DSMAILCOTCOP"]);    
                                                                    $erro     = "";
                                                                    $processo = "";
                                                                    $consultaFornLog = $this->service->getConsultaFornLog($ruptura["NRLOGRUPTURAOLAUT"], $ruptura["CDFORNECED"]);
                                                                    if(empty($consultaFornLog)){
                                                                        $this->service->insLogrupturaolautforn($ruptura["NRSEQRUPTURAOL"], $ruptura["CDFILIAL"], $processo, $erro, $fornecedor["CDFORNECEDENVIO"]);        
                                                                    }else{
                                                                        $this->service->atualizaLogrupturaolautForn($ruptura["NRLOGRUPTURAOLAUT"], $processo, $erro, $fornecedor["CDFORNECEDENVIO"]);                     
                                                                    }
                                                                    $af       = $fornecedor["NRAUTOFORN"];
                                                                    $fornecedorAF = $fornecedor["CDFORNECED"];
                                                                    $this->service->atualizaAFGeral($ruptura, $erro, $af, $fornecedorAF);
                                                                } catch (\Exception $e) {
                                                                    $isEmailSend = false;
                                                                    $erro = $e->getMessage();
                                                                    $erro = substr($erro, 0, 165);
                                                                    $consultaFornLog = $this->service->getConsultaFornLog($ruptura["NRLOGRUPTURAOLAUT"], $ruptura["CDFORNECED"]);
                                                                    if(empty($consultaFornLog)){
                                                                        $this->service->insLogrupturaolautforn($ruptura["NRSEQRUPTURAOL"], $ruptura["CDFILIAL"], $processo, $erro, $fornecedor["CDFORNECEDENVIO"]);        
                                                                    }else{
                                                                        $this->service->atualizaLogrupturaolautForn($ruptura["NRLOGRUPTURAOLAUT"], $processo, $erro, $fornecedor["CDFORNECEDENVIO"]);                     
                                                                    }
                                                                    $af       = $fornecedor["NRAUTOFORN"];
                                                                    $fornecedorAF = $fornecedor["CDFORNECED"];
                                                                    $this->service->atualizaAFGeral($ruptura, $erro, $af, $fornecedorAF);
                                                                }
                                                                if($isEmailSend) {
                                                                    $this->logEmail['SUCCESS'][] = array("MESSAGE" => "EMAIL_SEND_SUCCESS");
                                                                } else {
                                                                    $this->logEmail['ERROR'][] = array("MESSAGE" => "EMAIL_SEND_FAILED");
                                                                }
                                                                
                                                            } catch (\Exception $e) {
                                                            }
                                                        }else{
                                                            $erro     = 'FORN_SEM_EMAIL_CAD';
                                                            $consultaFornLog = $this->service->getConsultaFornLog($ruptura["NRLOGRUPTURAOLAUT"], $ruptura["CDFORNECED"]);
                                                            if(empty($consultaFornLog)){
                                                                $this->service->insLogrupturaolautforn($ruptura["NRSEQRUPTURAOL"], $ruptura["CDFILIAL"], $processo, $erro, $fornecedor["CDFORNECEDENVIO"]);        
                                                            }else{
                                                                $this->service->atualizaLogrupturaolautForn($ruptura["NRLOGRUPTURAOLAUT"], $processo, $erro, $fornecedor["CDFORNECEDENVIO"]);                     
                                                            }
                                                            $erro     = "Fornecedor ". $fornecedor["NRINSJURFORN"] . ' - ' . $fornecedor["NMFANTFORN"] . " sem e-mail  cadastrado.";
                                                            $af       = $fornecedor["NRAUTOFORN"];
                                                            $fornecedorAF = $fornecedor["CDFORNECED"];
                                                            $this->service->atualizaAFGeral($ruptura, $erro, $af, $fornecedorAF);
                                                        }    
                                                    }
                                                }    
                                            }else{
                                                $erro     = 'FORN_NAO_CAD_PORTAL';
                                                $consultaFornLog = $this->service->getConsultaFornLog($ruptura["NRLOGRUPTURAOLAUT"], $ruptura["CDFORNECED"]);
                                                if(empty($consultaFornLog)){
                                                    $this->service->insLogrupturaolautforn($ruptura["NRSEQRUPTURAOL"], $ruptura["CDFILIAL"], $processo, $erro, $fornecedor["CDFORNECEDENVIO"]);        
                                                }else{
                                                    $this->service->atualizaLogrupturaolautForn($ruptura["NRLOGRUPTURAOLAUT"], $processo, $erro, $fornecedor["CDFORNECEDENVIO"]);                     
                                                } 
                                                $erro     = "Fornecedor ". $fornecedor["NRINSJURFORN"] . ' - ' . $fornecedor["NMFANTFORN"] . " não cadastrado para acessar o portal.";
                                                $af       = $fornecedor["NRAUTOFORN"];
                                                $fornecedorAF = $fornecedor["CDFORNECED"];
                                                $this->service->atualizaAFGeral($ruptura, $erro, $af, $fornecedorAF);
                                            }
                                            
                                        }    
                                    }
                                }else{
                                    $erro     = 'CENTRAL_SEM_DADOS_EMAIL';
                                    $this->service->atualizaLogrupturaolaut($ruptura["NRLOGRUPTURAOLAUT"], $processo, $erro);
                                    $erro     = "Central de Compras ". $row1s["NMFILIAL"] . " sem cadastro para envio de e-mail.";
                                    $af       = "T";
                                    $fornecedorAF = "";
                                    $this->service->atualizaAFGeral($ruptura, $erro, $af, $fornecedorAF);
                                }
                                
                            } else{
                                $erro     = 'SEM_EMAIL_GESTOR';
                                $this->service->atualizaLogrupturaolaut($ruptura["NRLOGRUPTURAOLAUT"], $processo, $erro);
                                $erro     = "Central de Compras ". $row1s["NMFILIAL"] . " sem parametrização para gestor receber e-mail.";
                                $af       = "T";
                                $fornecedorAF = "";
                                $this->service->atualizaAFGeral($ruptura, $erro, $af, $fornecedorAF);
                            }    
                        }else{
                            $erro     = 'CENTAL_SEM_PARAM';
                            $this->service->atualizaLogrupturaolaut($ruptura["NRLOGRUPTURAOLAUT"], $processo, $erro);
                            $erro     = "Central de Compras ". $row1s["NMFILIAL"] . " sem parametrização para envio de e-mail.";
                            $af       = "T";
                            $fornecedorAF = "";
                            $this->service->atualizaAFGeral($ruptura, $erro, $af, $fornecedorAF);
                        }
                    }
                }    
            }    
        }
        return $filialOL;
    }
    
    
    private function tratarMensagemCorpoEmailForn($cdforneced, $cnpj, $nmFornecedor, $nrLote, $nrAutoforn) {
        $message = '';
        $msg     = '';
        $msg1    = '';  
        $attachments = [];
        $attOnBody = '';
        $clearAttachments = false;
        $msg  = " Prezado Fornecedor, ". $cnpj . " - ". $nmFornecedor  ."<br/>"."<br/>" ;  
        $msg1 = " Parabéns! Você foi selecionado em nosso processo de cotação."."<br/>".
                " A Autorização de Fornecimento foi disponibilizada em nosso Portal de Compras."."<br/>"."<br/>" ;  
        $af   = $nrAutoforn;
        if (strlen($af) > 15) {  
            $af1 = substr($af, 0, (strlen($af)-12));
            $af2 = ' e '.substr($af, -10); 
            $af  = $af1 . $af2;
            $msg2 = " Confira as AF´s  " . $af . " do Lote " . $nrLote . "." . "<br/>" . "<br/>" ;                      
            
                   
        } else {
            $msg2 = " Confira a AF " . $af . " do Lote " . $nrLote . "." . "<br/>" . "<br/>" ;                                
            
        }           
        $msg3 = " Lembramos que não são permitidas entregas em desacordo com a Autorização de Fornecimento. Sendo o fornecedor responsável por divergências ou possíveis devoluções." . "<br/>". "<br/>" ;   
                 
        $msg4 = "Verifique atentamente o local de entrega, dados de faturamento de cada unidade, produtos, marcas, preços, embalagens, quantidades e demais informações." . "<br/>". "<br/>" ;
    
        $msg5 = "Siga rigorosamente a Autorização de Fornecimento!" . "<br/>". "<br/>" ;

        $message =  $msg . $msg1 . $msg2 . $msg3 . $msg4 . $msg5;         
        $message = $attOnBody.$message;
        $message1 = "Email automático enviado pelo sistema Teknisa Operador Logístico";
        $message2 = "Disponilizado em: "; 
        $footer = "<div style=\"font-size: 10px; margin-top: 20px; word-wrap: break-word;\">" . $message1 . "<br/>" . $message2 . DateUtil::getDataAtual() . "</div>";
        $bodyMessage = "<table>
                            <tr>
                                <td class='description'>
                                    <br/>" . $message .
                                "</td>
                            </tr>
                            <tr class='footer'>
                                <td>
                                    <br> <p>" . $footer . "</p>
                                </td>
                            </tr>
                        </table>";
        return [$bodyMessage, $attachments, $clearAttachments];
    }
    
    
    public function sendMailFinal($stsenha, $stsmpt, $stlogin, $stemail, $cripto, $stendereco, $stmessage, $stassunto, $stenderecoCC) {
        //Mensagem padrÃ£o a ser enviada
        if (empty($stsenha)){
          $boautenticacao = false;    
        }else{
          $boautenticacao = true;  
        }; 
        $message = $stmessage;
        
        //Valida e "monta" a array dos emails
        $EMAILFOR = str_replace(";", ",", $stendereco);
        $EMAILFOR = str_replace(" ", "", $EMAILFOR);
        $EMAILFOR = explode(",", $EMAILFOR);
        
        //Valida e "monta" a array dos emails cÃ³pia
        if (!empty($stenderecoCC)){        
            $EMAILFORCC = str_replace(";", ",", $stenderecoCC);
            $EMAILFORCC = str_replace(" ", "", $EMAILFORCC);
            $EMAILFORCC = explode(",", $EMAILFORCC);
        }    
            
        //Define o Servidor Smtp
        $SERVIDORSMTP = explode(':', $stsmpt);
        $SERVIDORSMTP = $SERVIDORSMTP[0];
        
        //Define a porta Smtp
        $PORTSMTP = strstr($stsmpt, ':');
        if(strlen($PORTSMTP) > 1) {
            $PORTSMTP = explode(':', $PORTSMTP);
            $PORTSMTP = $PORTSMTP[1];
           
        }else{
            $PORTSMTP = "";    
        }
        //Valida o field Senha Smtp
        if (empty($stsenha)){
            $stsenha = '';
        } else {
            $stsenha =  self::getDelphiPasswordEncrypted($stsenha, false);
            //$stsenha = 'Davi041108**';
        }
        
        //Cria uma nova instancia de configuraÃ§Ãµes do Smtp
        $this->smtpUtil = new SMTP([
            'smtpAuth'      => $boautenticacao,
            'host'          => $SERVIDORSMTP,
            'username'      => $stemail,
            'port'          => $PORTSMTP,
            'smtpSecure'    => $cripto == 'TLS' ? 'TLS' :  ($cripto == 'SSL' ?  'SSL' : ''),
            'from'          => $stlogin,
            'subject'       => $message,
            'password'      => $stsenha
        ]);
        //Seta o assunto do email
        $this->smtpUtil->setSubject($stassunto);
        if ($stenderecoCC) {
            return $this->smtpUtil->sendMail($message, $EMAILFOR);
            //return $this->smtpUtil->sendMail($message, $EMAILFOR, $EMAILFORCC);
        } else {
            return $this->smtpUtil->sendMail($message, $EMAILFOR);
        }
    }
    
    public static function getDelphiPasswordEncrypted($password, $isEncrypt = true, $offset = '2') {
        $url = self::getAspUrlParameter() . '/comwebservice/service.asmx?wsdl';
        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );
        

        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            new \SoapVar($offset, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
        );
        
        $cryptType = $isEncrypt ? 'GetCripto' : 'GetUncripto';
        
        $params = array(
            'rdm'    => 'Bib.rdmDBGeral',
            'method' => $cryptType,
            'args'   => $args,
        );
        try {
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            throw new \Exception($e->getMessage());
        }
        return $encryption;
    }
    
    public static function getAspUrlParameter() {
        return InstanceManager::getInstance()->getParameter('aspUrl');
    }
    
    public function sendMailFinalGestor($stCdFilCentCmp) {
        $row1s = $this->service->getParasupri($stCdFilCentCmp); 
        if(!empty($row1s[0])) {
            foreach($row1s as $row1) {
                $senha    = $row1['CDSENHEMAILF']; 
                $smpt     = $row1['DSSMTPEMAILF']; 
                $login    = $row1['DSLOGINSMTP']; 
                $email    = $row1['DSEMAILSMTP']; 
                $cripto   = $row1['TIPOCRIPTO'];
                $nmFilial = $row1['NMFILIAL']; 
                $endereco = $row1['DSEMAILCONCLAUTRUPOL'];       
            }
            if((!empty($smpt)) && (!empty($login)) && (!empty($email)) && (!empty($endereco))) {
                
                $result           = self::tratarMensagemCorpoEmailFinal($row1);
                $message          = $result[0];
                $attachments      = $result[1];
                $clearAttachments = $result[2];
                if(!empty($message)){
                    $assunto          = "Relatório de Conclusão Automática do Processo de Migração de Ruptura do OL";
                    $messageSendMail  = $this->oplUtilities->messageSendMail($stCdFilCentCmp, $assunto, $message, 'TECFOOD'); 
                    try {
                        $isEmailSend = true;
                        self::sendMailFinal($senha, $smpt, $login, $email, $cripto, $endereco, $messageSendMail, $assunto, "");    
                    } catch (\Exception $e) {
                        $isEmailSend = false;
                    }
                }    
            }    
        }    
    }
    
    private function tratarMensagemCorpoEmailFinal($row1s) {
        $message = '';
        $msg     = '';
        $msg1    = '';  
        $attachments = [];
        $attOnBody = '';
        $bodyMessage = '';
        $clearAttachments = false;
        $todas    = "W";
        $results = $this->service->getRuptura($todas);
        foreach($results as $result) {
            $msg = " Operador Logístico:  " . $result["CDFILIAL"] . ' - ' . $result["NMFILIAL"] .
                   ' Ruptura: ' . $result['NRSEQRUPTURAOL']."<br/>";
            if ((empty($result["DSERRO"])) && (empty($result["DSPROCESSO"]))) {
                $msg1 = "Migração de Ruptura do OL concluída com sucesso."."<br/>"."<br/>" ;  
            } else if (($result["DSERRO"] == "NAO_HA_DADOS") && ($result["DSPROCESSO"] == "APURACAO")){
                $msg1 = "Migração de Ruptura do OL concluída com falha no processo de geração de AF."."<br/>"."<br/>";
            } else if ($result["DSPROCESSO"] == 'CONCLUSAO_RUPTURA_GERA_AF'){
                $msg1 = "Falha ao concluir o processo de migração de ruptura do OL com geração de AF."."<br/>".
                        "&nbsp;&nbsp;&nbsp;&nbsp;".'Erro: '.$result["DSERRO"]."<br/>"."<br/>";
            } else if (($result["DSERRO"] == 'INCONSISTENTE') && ($result["DSPROCESSO"] == 'GERACAO_AF')){    
                $af   = $result["LISTAAF"];
                if (strlen($af) > 15) {  
                    $af = substr($af, 0, (strlen($af)-2));
                    $af1 = substr($af, 0, (strlen($af)-12));
                    $af2 = ' e '.substr($af, -10); 
                    $af  = $af1 . $af2;
                    $msg1 = "Migração de Ruptura do OL concluída com AFs inconsistentes " . $af  .
                           " do fornecedor " . $result["NRINSJURFORN"] . ' - ' . $result["NMFANTFORN"] .'.'."<br/>"."<br/>" ; 
                } else {
                    $af = substr($af, 0, (strlen($af)-2));
                    $msg1 = "Migração de Ruptura do OL concluída com AF inconsistente " . $af  .
                           " do fornecedor " . $result["NRINSJURFORN"] . ' - ' . $result["NMFANTFORN"] .'.'."<br/>"."<br/>" ;                  
                }           
                
            } else if ($result["DSPROCESSO"] == "EMAIL"){
                switch ($result["DSERRO"]) {
                    case "CENTAL_SEM_PARAM":
                        $msg2 = "&nbsp;&nbsp;&nbsp;&nbsp;"."Central de Compras ". $row1s["NMFILIAL"] . " sem parametrização para envio de e-mail.";
                        break;
                    case "SEM_EMAIL_GESTOR":
                        $msg2 = "&nbsp;&nbsp;&nbsp;&nbsp;"."Central de Compras ". $row1s["NMFILIAL"] . " sem parametrização para gestor receber e-mail.";
                        break;
                    case "CENTRAL_SEM_DADOS_EMAIL":
                        $msg2 = "&nbsp;&nbsp;&nbsp;&nbsp;"."Central de Compras ". $row1s["NMFILIAL"] . " sem cadastro para envio de e-mail.";
                        break;
                    case "FORN_NAO_CAD_PORTAL":
                        $msg2 = "&nbsp;&nbsp;&nbsp;&nbsp;"."Fornecedor ". $result["NRINSJURFORN"] . ' - ' . $result["NMFANTFORN"] . " não cadastrado para acessar o portal.";
                        break;
                    case "FORN_SEM_EMAIL_CAD":
                        $msg2 = "&nbsp;&nbsp;&nbsp;&nbsp;"."Fornecedor ". $result["NRINSJURFORN"] . ' - ' . $result["NMFANTFORN"] . " sem e-mail  cadastrado.";
                        break;  
                    default:
                        $msg2 = "&nbsp;&nbsp;&nbsp;&nbsp;"."Erro de envio de e-mail: ". $result["DSERRO"];
                }
                $msg1 = "Migração de Ruptura do OL concluída com falha de envio de e-mail."."<br/>".$msg2."<br/>"."<br/>" ;                  
            } else{
                if (!empty($result["DSERRO"])){
                    $msg1 = "Migração de Ruptura do OL concluída com falha no processo de geração de AF."."<br/>".
                            "&nbsp;&nbsp;&nbsp;&nbsp;"."Erro: ".$result['DSERRO']."<br/>"."<br/>";
                }            
            }
            if(empty($message)){
                $message = $msg . $msg1;
            }else{
                $messagelen = strlen($message); 
                $msglen     = strlen($msg . $msg1);
                if($messagelen >= $msglen){
                    $existMsg = strripos($message,$msg . $msg1);
                    if ($existMsg == false){
                        $message = $message . $msg . $msg1;    
                    }
                }else if($messagelen < $msglen){
                    $existMsg = strripos($message,$msg . $msg1);
                    if ($existMsg == false){
                        $message = $message . $msg . $msg1;    
                    }
                }    
            }
        }
        if (!empty($message)){
            $message = $attOnBody.$message;
            $message1 = "Email automático enviado pelo sistema Teknisa Operador Logístico";
            $message2 = "Disponilizado em: "; 
            $footer = "<div style=\"font-size: 10px; margin-top: 20px; word-wrap: break-word;\">" . $message1 . "<br/>" . $message2 . DateUtil::getDataAtual() . "</div>";
            
            $bodyMessage = "<table>
                                <tr>
                                    <td class='description'>
                                        <br/>" . $message .
                                    "</td>
                                </tr>
                                <tr class='footer'>
                                    <td>
                                        <br> <p>" . $footer . "</p>
                                    </td>
                                </tr>
                            </table>";
        }
        return [$bodyMessage, $attachments, $clearAttachments];
    }
}