<?php 

namespace Teknisa\OPL\Modules\DocumentosFiscais\Controllers;

/* Bibliotecas */
use NFePHP\NFe\MakeNFe;
use NFePHP\NFe\ToolsNFe;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;

/* Classes DF */
use Teknisa\OPL\Services\ManifestacaoDestinatarioService;
class ManifestacaoDestinatarioController {
    
    public function __construct(ManifestacaoDestinatarioService $manifestacaoDestinatarioService) {
        $this->manifestacaoDestinatarioService = $manifestacaoDestinatarioService;
        set_time_limit(0);
    }
    
    public function montaConfigManifestacao(Request\Filter $request, Response $response) {
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $row = $requestConditions[0]["value"];
        try {
            $content = $this->manifestacaoDestinatarioService->montaConfigManifestacao
            ($row['CDEMPRESA'], $row['NRORG'], $row['CDINSCESTA'], $row["NRINSJURIEST"]);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('config_manifestacao', array('config' => $content)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function sincronizarNfeManifestacao(Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $config = $requestConditions[0]["value"];
        $params = $requestConditions[1]["value"];

        try {
            $sincronizacao = $this->manifestacaoDestinatarioService->sincronizarNfeManifestacao($config, $params);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('sincronizarNfeManifestacao', array($sincronizacao)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function manifestarNfe(Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $notasMarcadas = $requestConditions[0]["value"];
        $config = $requestConditions[1]["value"];
        $evento = $requestConditions[2]["value"];
        $justificativa = $requestConditions[3]["value"];

        $manifestacao = $this->manifestacaoDestinatarioService->manifestarNfe($notasMarcadas, $config, $evento, $justificativa);
        
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('manifestarNfe', $manifestacao));
    }

    public function gerarXmlNfeManifestacao(Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $notas = $requestConditions[0]["value"];
        $config = $requestConditions[1]["value"];

        try {
            $xml = $this->manifestacaoDestinatarioService->gerarXmlNfeManifestacao($notas, $config);
            if($xml) {
                $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('gerarXmlNfeManifestacao', array($xml)));
            }
            else {
                $response->setError(new Response\Error("Não foi possivel gerar o arquivo XML da nota fiscal. Verifique seu certificado digital/ambiente e tente novamente.", 500));
            }
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getDefaultFilterManif(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        $defaultFilter = $this->manifestacaoDestinatarioService->getDefaultFilterManif($row);
        
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('defaultFilter', $defaultFilter));
    }
    
}