<?php

namespace Teknisa\OPL\Modules\DocumentosFiscais\Services;

use Doctrine\ORM\EntityManager;

use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Suprimentos\Entities\Itemautf;
use Teknisa\Model\Fiscal\Entities\Itemproduto;


use Teknisa\Libs\Util\NewCode;
use Teknisa\OPL\Modules\DocumentosFiscais\Factories\Dfs11000NfSaidaFactory;

class Dfs11000NfSaidaService {

    protected $entityManager;
    protected $novoCodigo;
    protected $environment;

    private $filial;
    private $lancamento;
    private $organizacao;
    private $lancamentoSaidaFactory;

    //TODO Remover parametros não usados
    public function __construct(EntityManager $entityManager, NewCode $novoCodigo, $environment, $filial, $lancamento, $organizacao) {
        $this->novoCodigo = $novoCodigo;
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->organizacao = $organizacao;
        $this->filial = $filial;
        $this->lancamento = $lancamento;

        //TODO nao instanciar a classe
        $this->lancamentoSaidaFactory = new Dfs11000NfSaidaFactory($entityManager);
    }

    public static function instanceService(
        EntityManager $entityManager, NewCode $novoCodigo, $environment, $filial, $lancamento, $organizacao = null
    ) {
        if (!$organizacao)
            $organizacao = $environment->getNrOrgTrab();

        return new self($entityManager, $novoCodigo, $environment, $filial, $lancamento, $organizacao);
    }

    private function calcularQuantidadeAF($item){
        $qtProgramadaAF = $this->lancamentoSaidaFactory->quantidadeProgramadaAF($this->filial, $this->lancamento, $item->getVrunititem(), $item->getCdproduto());

        $qtAF = 0;
        foreach ($qtProgramadaAF as $af) {
            $qtAF += $af['QTPROGENTAF'];
        }

        return array('af' => $qtProgramadaAF, 'quantidade' => $qtAF);
    }

    private function calcularQuantidadeEntradaAF($item){
        $qtEntradaAF = $this->lancamentoSaidaFactory->quantidadeEntradaAF($this->filial, $this->lancamento, $item->getVrunititem(), $item->getCdproduto());

        $qtNFentrada = 0;
        foreach ($qtEntradaAF as $af) {
            $qtNFentrada += $af['QTITEMPROD'];
        }

        return $qtNFentrada;
    }

    private function calcularQuantidadeDevolvidaAF($item){
        $qtDevolvidaAF = $this->lancamentoSaidaFactory->quantidadeDevolvidaAF($this->filial, $this->lancamento, $item->getVrunititem(), $item->getCdproduto());

        $qtDevolvida = 0;
        foreach ($qtDevolvidaAF as $af) {
            $qtDevolvida += $af['QTITEMPROD'];
        }

        return $qtDevolvida;
    }

    private function calcularAcrescimoDecrescimo($qtNFentrada, $qtAF, $qtDevolvida){
        if($qtNFentrada > ($qtAF + $qtDevolvida)){
            $acrescimo = $qtNFentrada - ($qtAF + $qtDevolvida);
            $decrescimo = 0;
        }
        else{
            $acrescimo = 0;
            $decrescimo = $qtNFentrada - ($qtAF + $qtDevolvida);
        }

        return array('acrescimo' => $acrescimo, 'decrescimo' => $decrescimo);
    }

    private function calcularQuantidadeDevolucao($item, $acrescimo){
        if($item->getQtitemprod() <= $acrescimo)
            return 0;
        else
            return $item->getQtitemprod() - $acrescimo;
    }

    private function buscarItemAF($af){
        return $this->entityManager->getRepository('Teknisa\Model\Suprimentos\Entities\Itemautf')->findOneBy(
            array('nrorg' => $this->organizacao, 'cdfildest' => $af['CDFILDEST'],
                'cdfillanc' => $af['CDFILLANC'], 'nrautoforn' => $af['NRAUTOFORN'],
                'cdmarca' => $af['CDMARCA'], 'cdproduto' => $af['CDPRODUTO'],
                'dtprogentaf' => DateUtil::getDataDeString($af['DTPROGENTAF'], DateUtil::FORMATO_BRASILEIRO_DATAHORA)));
    }

    private function verificarExisteAF($produto){
        return $this->lancamentoSaidaFactory->existeAF($this->filial, $this->lancamento, $produto);
    }

    private function verificarExisteAFXML($produto){
        return $this->lancamentoSaidaFactory->existeAFXML($this->filial, $this->lancamento, $produto);
    }

    private function atualizarItensAF($operador){
        $itensNota = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Itemproduto')->findBy(
            array('nrorg' => $this->organizacao, 'cdfilial' => $this->filial, 'nrlanctonf' => $this->lancamento));

        foreach ($itensNota as $item) {
            $this->lancamentoSaidaFactory->callProcedure_ATUALIZA_QTRELAC_ITNFORIGEM($this->filial, $this->lancamento, $item->getNrseqitprod());

            if($this->verificarExisteAF($item->getCdproduto())){
                $arrayAF = $this->calcularQuantidadeAF($item);
                $qtProgramadaAF = $arrayAF['af'];
                $qtAF = $arrayAF['quantidade'];
                $qtNFentrada = $this->calcularQuantidadeEntradaAF($item);
                $qtDevolvida = $this->calcularQuantidadeDevolvidaAF($item);

                $arrayAcrescimoDecrescimo = $this->calcularAcrescimoDecrescimo($qtNFentrada, $qtAF, $qtDevolvida);
                $acrescimo = $arrayAcrescimoDecrescimo['acrescimo'];
                $decrescimo = $arrayAcrescimoDecrescimo['decrescimo'];

                $qtDevolucao = $this->calcularQuantidadeDevolucao($item, $acrescimo);
                foreach ($qtProgramadaAF as $af) {
                    if($qtDevolucao > 0){
                        /*-----------NOTA SENDO ALTERADA-----------*/
                        if($af['QTEFETENTRAF'] == 0){ //af em aberto
                            if(($qtDevolucao + $qtDevolvida) < $af['QTPROGENTAF']){
                                $status = 1; //baixa parcial
                                $baixaFutura = '';
                                if($qtDevolvida > 0){
                                    if($qtNFentrada < ($qtAF + $qtDevolvida)){
                                        $qtEntregue = ($af['QTPROGENTAF'] + ($qtNFentrada - ($qtAF + $qtDevolvida))) - ($qtDevolucao + $qtDevolvida);
                                        $qtNEntregue = $af['QTPROGENTAF'] - $qtEntregue;
                                    }
                                    else{
                                        $qtEntregue = $af['QTPROGENTAF'] - ($qtDevolucao + $qtDevolvida);
                                        $qtNEntregue = $af['QTPROGENTAF'] - $qtEntregue;
                                    }
                                    if($qtEntregue == 0)
                                        $status = ''; //Em aberto
                                }
                                else {
                                    if($qtNFentrada - ($qtAF + $qtDevolvida) < 0){
                                        $qtEntregue = ($af['QTPROGENTAF'] + ($qtNFentrada - ($qtAF + $qtDevolvida))) - $qtDevolucao;
                                        $qtNEntregue = $af['QTPROGENTAF'] - $qtEntregue;
                                    }
                                    else{
                                        $qtEntregue = $af['QTPROGENTAF'] - $qtDevolucao;
                                        $qtNEntregue = $af['QTPROGENTAF'] - $qtEntregue;
                                    }
                                    if($qtEntregue == 0)
                                        $status = ''; //Em aberto
                                }
                            }
                            else{
                                $status = ''; //Em aberto
                                $baixaFutura = 'X';
                                if($qtDevolvida > 0){
                                    if($qtNFentrada - ($qtAF + $qtDevolvida) < 0){
                                        if(($qtDevolucao + $qtDevolvida) > ($af['QTPROGENTAF'] + ($qtNFentrada - ($qtAF + $qtDevolvida)))){
                                            $qtEntregue = 0;
                                            $qtNEntregue = $af['QTPROGENTAF'] - $qtEntregue;
                                            $qtDevolucao = $qtDevolucao + $qtDevolvida;
                                        }
                                        else{
                                            $qtEntregue = ($af['QTPROGENTAF'] + ($qtNFentrada - ($qtAF + $qtDevolvida))) - ($qtDevolucao + $qtDevolvida);
                                            $qtNEntregue = $af['QTPROGENTAF'] - $qtEntregue;
                                        }
                                    }
                                    else{
                                        if(($qtDevolucao + $qtDevolvida) > $af['QTPROGENTAF']){
                                            $qtEntregue = 0;
                                            $qtNEntregue = $af['QTPROGENTAF'] - $qtEntregue;
                                        }
                                        else{
                                            $qtEntregue = $af['QTPROGENTAF'] - ($qtDevolucao + $qtDevolvida);
                                            $qtNEntregue = $af['QTPROGENTAF'] - $qtEntregue;
                                        }
                                    }
                                }
                                else {
                                    if($qtDevolucao >= $af['QTPROGENTAF']){
                                        $qtEntregue = 0;
                                        $qtNEntregue = $af['QTPROGENTAF'];
                                    }
                                    else{
                                        if(($qtNFentrada - ($qtAF + $qtDevolvida)) < 0){
                                            $qtEntregue = ($af['QTPROGENTAF'] - ($qtNFentrada - ($qtAF + $qtDevolvida))) - $qtDevolucao;
                                            $qtNEntregue = $af['QTPROGENTAF'] - $qtEntregue;
                                        }
                                        else{
                                            $qtEntregue = $af['QTPROGENTAF'] - $qtDevolucao;
                                            $qtNEntregue = $af['QTPROGENTAF'] - $qtEntregue;
                                        }
                                    }
                                }
                            }
                            if($qtEntregue == 0){
                                $status = ''; //Em aberto
                                $baixaFutura = 'X';
                            }
                        }
                        else {
                            if($qtDevolvida > 0){
                                if(($qtDevolucao + $qtDevolvida) == $af['QTPROGENTAF']){
                                    $status = ''; //Em aberto
                                    $baixaFutura = 'X';
                                }
                                else{
                                    $status = '1'; //Baixa parcial
                                    $baixaFutura = '';
                                }

                                if(($af['QTPROGENTAF'] - ($qtDevolucao + $qtDevolvida)) > 0){
                                    $qtEntregue = $af['QTPROGENTAF'] - ($qtDevolucao + $qtDevolvida);
                                    $qtNEntregue = $af['QTPROGENTAF'] - $qtEntregue;
                                }
                                else{
                                    $qtEntregue = 0;
                                    $qtNEntregue = $af['QTPROGENTAF'];
                                }
                            }
                            else {
                                if($qtDevolucao == $af['QTPROGENTAF']){
                                    $status = ''; //Em aberto
                                    $baixaFutura = 'X';
                                }
                                else{
                                    $status = '1'; //Baixa parcial
                                    $baixaFutura = '';
                                }

                                if(($af['QTPROGENTAF'] - $qtDevolucao) > 0){
                                    $qtEntregue = $af['QTPROGENTAF'] - $qtDevolucao;
                                    $qtNEntregue = $af['QTPROGENTAF'] - $qtEntregue;
                                }
                                else{
                                    $qtEntregue = 0;
                                    $qtNEntregue = $af['QTPROGENTAF'];
                                }
                            }
                            if($qtEntregue == 0){
                                $status = ''; //Em aberto
                                $baixaFutura = 'X';
                            }
                        }

                        /*-----------NOTA SENDO INCLUIDA-----------*/
                        // if($qtDevolucao >= $af['QTEFETENTRAF']){
                        //     $status = ''; //Em aberto
                        //     $baixaFutura = 'X';
                        // }
                        // else{
                        //     $status = '1'; //Baixa parcial
                        //     $baixaFutura = '';
                        // }

                        // if(($af['QTEFETENTRAF'] - $qtDevolucao) > 0){
                        //     $qtEntregue = $af['QTEFETENTRAF'] - $qtDevolucao;
                        //     $qtNEntregue = $af['QTPROGENTAF'] - $qtEntregue;
                        // }
                        // else{
                        //     $qtEntregue = 0;
                        //     $qtNEntregue = $af['QTEFETENTRAF'];
                        // }

                        // if($status != '' && ($qtDevolucao + $qtDevolvida) >= $qtNFentrada){
                        //     $status = ''; //Em aberto
                        //     $baixaFutura = 'X';
                        //     $qtEntregue = 0;
                        //     $qtNEntregue = $af['QTEFETENTRAF'];
                        // }

                        // if($qtEntregue == 0){
                        //     $status = ''; //Em aberto
                        //     $baixaFutura = 'X';
                        // }
                        /*END--------NOTA SENDO INCLUIDA-----------*/

                        $this->lancamentoSaidaFactory->updateItemAlteracaoNotaAF($this->organizacao, $operador, $this->buscarItemAF($af), $status, $qtEntregue, $baixaFutura, 'S');

                        //Atualiza QtDevolvida
                        if($qtDevolucao > $qtDevolvida)
                            $qtDevolvida = 0;
                        else
                            $qtDevolvida = $qtDevolvida - $qtDevolucao;

                        $qtDevolucao = $qtDevolucao - $qtNEntregue;
                    }
                    else{
                        $status = '2';
                        if($decrescimo < 0 && $qtDevolucao == 0){
                            $status = '1'; //Baixa parcial
                            $qtEntregue = ($af['QTPROGENTAF'] + $decrescimo) - abs($qtDevolucao);
                            $qtNEntregue = $af['QTPROGENTAF'] - $qtEntregue;
                        }
                        else{
                            $qtEntregue = $af['QTPROGENTAF'];
                            $qtNEntregue = 0;
                        }

                        if($qtEntregue == 0)
                            $status = ''; //Em aberto
                        $baixaFutura = '';

                        $iddevolucaonfit = ($decrescimo < 0 && $qtDevolucao == 0) ? 'S' : 'N';
                        $this->lancamentoSaidaFactory->updateItemAlteracaoNotaAF($this->organizacao, $operador, $this->buscarItemAF($af), $status, $qtEntregue, $baixaFutura, $iddevolucaonfit);

                        //Atualiza QtDevolvida
                        $qtDevolucao = 0;
                        $decrescimo = 0;
                    }
                }
            }
        }
    }

    private function atualizaQuantidadeItemExclusaoNotaAF($qtProgramadaAF, $qtDevolvida, $operador){
        foreach ($qtProgramadaAF as $af) {
            $qtNEntregue = $af['QTPROGENTAF'] - $af['QTEFETENTRAF'];
            //se quantidade da nota de devolução é maior que a quantidade não entregue da af
            if($qtDevolvida >= $qtNEntregue){
                //atualiza a quantidade da af (qt entregue := qt entregue + qt nao entregue)
                $this->lancamentoSaidaFactory->updateQuantidadeItemNotaAF($this->organizacao, $operador, $this->buscarItemAF($af), $qtNEntregue, 'N');
                //deixa o valor da quatidade a ser devolvida para a proxima af
                $qtDevolvida -= $qtNEntregue;
            }
            // se a nota não estiver com baixa total(ou seja a devolução não alterou o status da af na inclusão)
            elseif($af['IDBAIXITMAF'] != '2'){
                $iddevolucaonfit = $qtDevolvida < ($af['QTEFETENTRAF'] + $qtDevolvida) ? 'N' : 'S';
                //atualiza a quantidade da af (qt entregue := qt entregue + qt da nota de devolucao)
                $this->lancamentoSaidaFactory->updateQuantidadeItemNotaAF($this->organizacao, $operador, $this->buscarItemAF($af), $qtDevolvida, 'N');
                $qtDevolvida = 0;
            }
        }

        return $qtDevolvida;
    }

    private function atualizarQuantidadeItensAFExclusao($operador){
        $itensNota = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Itemproduto')->findBy(
            array('nrorg' => $this->organizacao, 'cdfilial' => $this->filial, 'nrlanctonf' => $this->lancamento));

        foreach ($itensNota as $item) {
            if($this->verificarExisteAF($item->getCdproduto())){
                $qtDevolvida = $item->getQtitemprod();
                //verifica se o item da nota de devolução esta relacionado a um documento de origem que veio de af
                $qtProgramadaAF = $this->lancamentoSaidaFactory->quantidadeProgramadaAF($this->filial, $this->lancamento, $item->getVrunititem(), $item->getCdproduto());
                $qtDevolvida = $this->atualizaQuantidadeItemExclusaoNotaAF($qtProgramadaAF, $qtDevolvida, $operador);
            }
            //verifica se o item da nota de devolução esta relacionado a um documento de origem que veio de af
            elseif($this->verificarExisteAFXML($item->getCdproduto())){
                $qtDevolvida = $item['QTITEMPROD'] * $item['FATORCONV'];
                $qtProgramadaAFXML = $this->lancamentoSaidaFactory->quantidadeProgramadaAFXML($this->filial, $this->lancamento, $item->getVrunititem(), $item->getCdproduto());
                $qtDevolvida = $this->atualizaQuantidadeItemExclusaoNotaAF($qtProgramadaAFXML, $qtDevolvida, $operador);
            }
            else{
                $qtDevolvida = $item->getQtitemprod();
                //verifica se o item da nota de devolução esta relacionado a um documento de origem que veio de af
                $qtProgramadaAF = $this->lancamentoSaidaFactory->quantidadeProgramadaAF($this->filial, $this->lancamento, $item->getVrunititem(), $item->getCdproduto());
                $qtDevolvida = $this->atualizaQuantidadeItemExclusaoNotaAF($qtProgramadaAF, $qtDevolvida, $operador);
            }
        }
    }

    private function atualizarBaixaItensAFExclusao($operador){
        $afOrigem = $this->lancamentoSaidaFactory->buscaItensAForigem($this->filial, $this->lancamento);
        foreach ($afOrigem as $af) {
            $itemautf = $this->buscarItemAF($af);
            if($itemautf){
                if($itemautf->getQtefetentraf() >= $itemautf->getQtprogentaf()){
                    //baixa total
                    $this->lancamentoSaidaFactory->updateBaixaItemNotaAF($this->organizacao, $operador, $itemautf, '2', 'X');
                }
                else{
                    if($itemautf->getQtefetentraf() == 0){
                        //Em aberto
                        $this->lancamentoSaidaFactory->updateBaixaItemNotaAF($this->organizacao, $operador, $itemautf, '', '');
                    }
                    else{
                        //baixa parcial
                        $this->lancamentoSaidaFactory->updateBaixaItemNotaAF($this->organizacao, $operador, $itemautf, '1', '');
                    }
                }
            }
        }
    }

    private function atualizarStatusAFNForigem(){
        $afOrigem = $this->lancamentoSaidaFactory->buscaAForigem($this->filial, $this->lancamento);
        foreach ($afOrigem as $af) {
            $this->lancamentoSaidaFactory->callProcedure_BAIXAAF($af['CDFILDEST'], $af['CDFILLANC'], $af['NRAUTOFORN']);
        }
    }

    public function atualizarStatusAFdevolucaoAlteracao($idoperdevonf){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $this->atualizarItensAF($this->environment->getCdOperador());
            if($idoperdevonf)
                $this->atualizarStatusAFNForigem();

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public function atualizarStatusAFdevolucaoExclusao(){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $operador = $this->environment->getCdOperador();

            $this->atualizarQuantidadeItensAFExclusao($operador);
            $this->atualizarBaixaItensAFExclusao($operador);
            $this->atualizarStatusAFNForigem();

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }
}
