<?php
namespace Teknisa\OPL\Modules\Relatorios\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\OPL\Modules\Relatorios\Services\CalcularService;
use Teknisa\OPL\Util\OplUtilities;
use Zeedhi\Framework\ORM\DateTime;
use Teknisa\Libs\Util\DateUtil;


class CalcularController extends \Teknisa\GER\Controllers\TeknisaCrud 
{
    private $errors = array();
    
    public function __construct($crudService, ParameterBag $parameterBag, $connection, CalcularService $service, $dataSourceManager, $environment)
    {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->connection = $connection;
        $this->service = $service;
        $this->dataSourceManager = $dataSourceManager;
		$this->environment = $environment;
    }
    
    public function compValidationListagemCorte(DTO\Request\Row $request, DTO\Response $response) {
        try{
            $row = $request->getRow();
            $passwordAut['PARAM'] = $this->verificaUsoDeSenhaOperadorRelacionado($row) ? 'S' : 'N';
            $response->addDataSet(new DataSet('passwordAut', $passwordAut));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function passwordValidation(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $dataSourceName = 'operador';
        $conditions = array(array('columnName' => 'CDOPERADOR', 'operator' => '=', "value" => $row['CDOPERADOR']));
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        if (md5($row['PASSWORD']) != $dataSet[0]['CDSENHAOPERWEB']) {
            $response->setError(new Error("Senha inválida", 1));
        }
    }
    
    private function verificaUsoDeSenhaOperadorRelacionado($row) {
        $operRelacResult = $this->service->qryOperRelacAux(array('P_CDCOMPRADOR' => $row['CDCOMPRADOR']));
        $adqParamResult = $this->service->adqParam(array('CDFILCENTCMP' => $row['CDCENTCMP']));
        
        if (empty($operRelacResult)) {
            if ($adqParamResult[0]['IDSISDOS'] != 3) {
                return true;
            }
        } else {
            $operadorComprador = $this->service->adqOperadorComprador(array('P_CDCOMPRADOR' => $row['CDCOMPRADOR'], 'P_CDOPERADOR'  => $row['CDOPERADOR']));
            
            $this->isOperadorRelacionado($operadorComprador);
            $this->isOperadorAutorizadoRealizarApuracao($operadorComprador);
            
            if (!empty($operadorComprador)) {
                return true;
            } else {
                return false;
            }
        }
        
        return false;
    }
    
    //Executa as Procedures "GERA_SOLICITA_CORTE_OL" e "APURA_EFETIVACAO"
    public function getGeraCorteOL(DTO\Request\Row $request, DTO\Response $response) {
        //Referencia a row
        $row = $request->getRow();
        
        //Busca o parametro/código do NRLOGCORTEOL
        $NRLOGCORTEOL = $this->service->generateCode('LOGCORTEOL');
        
        //Validações para o tipo de consulta
        if($row['P_DTFINAL']['end'] !== '' && $row['P_DTFINALAF']['end'] !== '') {
            $row['P_TIPOCONSULTA'] = '3';
        } else if ($row['P_DTFINAL']['end'] !== '' ) {
            $row['P_TIPOCONSULTA'] = '2';
        } else if ($row['P_DTFINALAF']['end'] !== '') {
            $row['P_TIPOCONSULTA'] = '1';
        }

        //Modifica os parametros de data de acordo com o tipo de consulta
        if ($row['P_TIPOCONSULTA'] === "1") {
            $row['P_DTINICIAL']['start'] = '';
            $row['P_DTFINAL']['end'] = '';
        } else if ($row['P_TIPOCONSULTA'] === "2") {
            $row['P_DTINICIALAF']['start'] = '';
            $row['P_DTFINALAF']['end'] = '';
        }
        
        //Array que armazena os parametros para a procedure "GERA_SOLICITA_CORTE_OL"
        $arrayGeraSolicitaCorte = [
            'P_CDFILIOPELOG' => $row['P_CDFILIOPELOG'],
            'P_CDFILLANC' => $row['P_CDFILLANC'],
            'P_CDOPERLANC' => $row['P_CDOPERLANC'],
            'P_DTINICIAL' => $row['P_DTINICIAL'],
            'P_DTFINAL' => $row['P_DTFINAL'],
            'P_DTINICIALAF' => $row['P_DTINICIALAF'],
            'P_DTFINALAF' => $row['P_DTFINALAF'],
            'P_TIPOCONSULTA' => $row['P_TIPOCONSULTA'],
            'P_COMPRADOR' => $row['P_COMPRADOR'],
            'P_CDREGIAO' => $row['P_CDREGIAO'],
            'P_CDPRODINIC' => $row['P_CDPRODINIC'],
            'P_CDPRODFIN' => $row['P_CDPRODFIN'],
            'P_PRXSOLICMP' => '',
            'P_NRLOGCORTEOL' => $NRLOGCORTEOL
        ];

        //Executa a procedure "GERA_SOLICITA_CORTE_OL"
        $getSolicitaCorteOL = $this->service->proceduregetSolicitaCorteOL($arrayGeraSolicitaCorte);
        
        //Armazena o resultado da procedure "GERA_SOLICITA_CORTE_OL"
        $result = substr($getSolicitaCorteOL['P_ROWSAFFECT'], 10, 2);
        
        //Verfica se o OUTPUT da procedure "GERA_SOLICITA_CORTE_OL" retornou "NN"
        if (substr($getSolicitaCorteOL['P_ROWSAFFECT'], 10, 2) !== "NN") {

            //Define o P_NRSEQAPUR
            $NRSEQAPUR = $this->service->generateCode('APURPRECO' . $row['P_CDCENTCMP']); //CODIGO GERADO
            
            //Seta o Produto Inicial
            $CDPRODINI = $this->service->getProdIni(array('P_CDPRODINIC' => $row['P_CDPRODINIC'], 'P_CDPRODFIN' => $row['P_CDPRODFIN']));
            
            //Seta o Produto Final
            $CDPRODFIN = $this->service->getProdFin(array('P_CDPRODINIC' => $row['P_CDPRODINIC'], 'P_CDPRODFIN' => $row['P_CDPRODFIN']));

            //Valida a Região
            if ($row['P_CDREGIAO'] !== 'T') {
                $CDRegiao = $row['P_CDREGIAO'];
            } else {
                $CDRegiao = NULL;
            }
            
            //Define o CDOPERADOROL
            $CDOperadorOL = "_".$row['P_CDFILIOPELOG']."_";
            
            //Array que armazena os parametros para a procedure "APURA_EFETIVACAO"
            $arrayApuraEfetivacao = [
                'P_CDCENTCMP' => $row['P_CDCENTCMP'],
                'P_NRSEQAPUR' => $NRSEQAPUR,
                'P_PRODINI' => $CDPRODINI[0]["CDARVPROD"],
                'P_PRODFIN' => $CDPRODFIN[0]["CDARVPROD"],
                'P_DTINI' => $row['P_DTINICIALAF'],
                'P_DTFIN' => $row['P_DTFINALAF'],
                'P_CDOPERADOR' => $row['P_COMPRADOR'],
                'P_CDREGIAO' => $CDRegiao,
                'P_CDFILIAL' => 'T',
                'P_DTPROCESSA' => $row['P_DTPROCESSA'],
                'P_FILIPROC' => $row['P_CDFILLANC'],
                'P_CDGRUPFILI' => '',
                'P_OPTIMIZER_MODE' => 'ALL_ROWS',
                'P_CDOPERLANC' => $row['P_CDOPERLANC'],
                'P_CDCOMPRADOR' => $row['P_COMPRADOR'],
                'P_LISTAFILIAL' => $CDOperadorOL,
                'P_EXTRA' => 'N',
                'P_NRLOGCORTEOL' => $NRLOGCORTEOL,
                'P_IDQTFRAC' => 'N',
                'P_IDUTLITEMRUPTURA' => 'N'
            ];
            
            //Executa a procedure "APURA_EFETIVACAO"
            $apuraEfetivacao = $this->service->procedureApuraEfetivacao($arrayApuraEfetivacao);
            
            //Seta o NrLogCorteOL como int
            $verificaCorteOL = intval($NRLOGCORTEOL);
            
            //Verifica se NrLogCorteOL é = vazio/0
            if (empty($verificaCorteOL)) {
                $result = [
                    ['P_ROWSAFFECT' => $getSolicitaCorteOL['P_ROWSAFFECT'].'N'],
                    ['P_NRSEQAPUR' => $NRSEQAPUR]
                ];
            } else {
                $result = [
                    ['P_ROWSAFFECT' => $getSolicitaCorteOL['P_ROWSAFFECT'].'S'],
                    ['P_NRSEQAPUR' => $NRSEQAPUR]
                ];
            }
            
        } else {
            $result = 'NN';
        }
        
        //Retorna os dados
        $response->addDataSet(new DataSet("result", [$result]));
    }
    
    private function isOperadorAutorizadoRealizarApuracao($operadorComprador) {
        if ($operadorComprador[0]['IDAPURCMPOPER'] == 'N') {
            throw new \Exception("Operador não autorizado a realizar apuração para o comprador.", 1);
        }
        
        return true;
    }
    
    private function isOperadorRelacionado($operadorComprador) {
        if (empty($operadorComprador)) {
            throw new \Exception("Operador não relacionado ao comprador.", 1);
        }
    
        return true;
    }
    
}