<?php

namespace Teknisa\OPL\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\ORM\DateTime;

use Teknisa\OPL\DocumentosFiscais\Factories\ReplicacaoDocumentosFactory;
use Teknisa\OPL\DocumentosFiscais\Factories\ProceduresNfFactory;
use Teknisa\OPL\Services\ValidacoesEstoqueService;

use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Fiscal\Entities\Notafiscal;
use Teknisa\Model\Fiscal\Entities\Serieentr;
use Teknisa\Model\Geral\Entities\Filial;
use Teknisa\Model\Geral\Entities\Paramfilial;
use Teknisa\Model\Geral\Entities\Paramgeral;
use Teknisa\Model\Geral\Entities\Tipooperac;
use Estoque\Entities\Lanctoestoq;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class ReplicacaoDocumentosService {

    protected $entityManager;
    protected $novoCodigo;
    protected $environment;
    protected $replicacaoDocumentosFactory;
    protected $proceduresNfFactory;
    
    private $geraReplicacaoAutomatica = false;
    private $notafiscal;
    private $dataProcessamento;
    private $replicacaoManual;
    private $tipooperac;
    private $paramgeral;
    private $arrayErrosReplicacao = array();
    private $stringErroReplicacao = '';
    private $notasDeletadas = true;
    
    const NOVO_CODIGO_NOTAFISCAL = 'LANCNF/FILIAL';
    const NOVO_CODIGO_ESTOQUE = 'LANCTOEST';

    public function __construct(ValidacoesEstoqueService $validacoesEstoqueService, EntityManager $entityManager, NewCode $novoCodigo, $environment, $notafiscal, $dataProcessamento, $replicacaoManual = false)
    {
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->entityManager = $entityManager;
        $this->novoCodigo = $novoCodigo;
        $this->environment = $environment;
        $this->notafiscal = $notafiscal;
        $this->dataProcessamento = $dataProcessamento;
        $this->replicacaoManual = $replicacaoManual;
        $this->replicacaoDocumentosFactory = new ReplicacaoDocumentosFactory($entityManager);
        $this->proceduresNfFactory = new ProceduresNfFactory($entityManager);
    }

    public function replicar($documento = array(), $itens = array()) {
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $tnrreplicdoc = $this->replicacaoManual ? 'M' : 'A';
            $nrreplicdoc  = $this->replicacaoManual ? $documento['NRREPLICDOC'] : ' ';
            
            $nrregistro = 0;
            
            $documentos = $this->replicacaoDocumentosFactory->getDocumentos($this->notafiscal->getCdfiliemit(), $this->notafiscal->getCdtipooper(), $tnrreplicdoc, $nrreplicdoc);
            if($documentos){
                if($this->verificarUtilizacaoNumeracaoTransferencia()){
                    $this->verificarFilialDestino();
                    foreach ($documentos as $doc) {
                        if(!$this->replicacaoManual) $itens = $this->replicacaoDocumentosFactory->getItensDocumento($this->notafiscal->getCdfiliemit(), $doc['NRREPLICDOC']);
                        if($itens){
                            foreach ($itens as $item) {
                                if($item['CDFILIORIGRD'] == $this->notafiscal->getCdfiliemit() && $item['CDFILIDESTRD'] == $this->notafiscal->getCdfilirece()){
                                    if($this->verificarDuplicidadeDeNotas($this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf(), $item['CDFILIORIGRD'], 
                                                $item['CDFILIDESTRD'], $item['IDENTRSAIDOP'], $item['CDTIPOOPER'], 
                                                empty($item['CDALMOXARIFE']) ? 'P' : $item['CDALMOXARIFE'], empty($item['CDLOCALESTOQ']) ? 'P' : $item['CDLOCALESTOQ'])){
                                        $filialMovimentacao = $item['IDENTRSAIDOP'] === 'E' ? $item['CDFILIDESTRD'] : $item['CDFILIORIGRD'];
                                        if($this->verificarData($this->notafiscal->getDtentrsaid(), $filialMovimentacao)){
                                            $this->replicacaoDocumentosFactory->inserirReplicacaoTemporaria(
                                                $this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf(), $item['CDFILIORIGRD'], 
                                                $item['CDFILIDESTRD'], $item['IDENTRSAIDOP'], $item['CDTIPOOPER'], $item['CDSERIE'], 
                                                empty($item['CDALMOXARIFE']) ? 'P' : $item['CDALMOXARIFE'], empty($item['CDLOCALESTOQ']) ? 'P' : $item['CDLOCALESTOQ'], 
                                                $this->environment->getNrOrgTrab(), $this->environment->getCdOperador());
                                        }
                                    }
                                }
                                
                                $nrregistro++;
                            }
                        }
                        else if ($this->notafiscal->getIdorigdestnf() === 'L'){
                            $operacaoEntrada = $this->verificarExisteOperacaoEntrada();
                            $serieEntrada = $this->verificarExisteSerieEntrada();
                            if($this->verificarDuplicidadeDeNotas($this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf(), $this->notafiscal->getCdfiliemit(), 
                                        $this->notafiscal->getCdfilirece(), 'E', $this->notafiscal->getCdtipooper(), 'P', 'P')){
                                if($operacaoEntrada && $serieEntrada){
                                    if($this->verificarData($this->notafiscal->getDtentrsaid(), $this->notafiscal->getCdfilirece())){
                                        $this->replicacaoDocumentosFactory->inserirReplicacaoTemporaria(
                                            $this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf(), $this->notafiscal->getCdfiliemit(), 
                                            $this->notafiscal->getCdfilirece(), 'E', $this->notafiscal->getCdtipooper(), $this->notafiscal->getCdserie(), 'P', 'P', 
                                            $this->environment->getNrOrgTrab(), $this->environment->getCdOperador());
                                    }
                                }
                            }
                        }
                    }
                }
            }
            else{
                $replicacoes = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Notafiscal')->findBy(
                                                                                array('iddocreplic' => $this->notafiscal->getCdfilial() . $this->notafiscal->getNrlanctonf()));
                foreach ($replicacoes as $replicacao) {
                    $serieEntrada = $this->verificarExisteSerieEntrada();
                    if($serieEntrada){
                        if($this->verificarDuplicidadeDeNotas($this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf(), $replicacao['CDFILIEMIT'], 
                                                $replicacao['CDFILIRECE'], $replicacao['IDENTRSAIDOP'], $this->notafiscal->getCdtipooper(), 'P', 'P')){
                            $this->replicacaoDocumentosFactory->inserirReplicacaoTemporaria(
                                            $this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf(), $replicacao['CDFILIEMIT'], 
                                            $replicacao['CDFILIRECE'], $replicacao['IDENTRSAIDOP'], $this->notafiscal->getIdentrsaidop(), 
                                            $this->notafiscal->getCdtipooper(), $this->notafiscal->getCdserie(), 'P', 'P', 
                                            $this->environment->getNrOrgTrab(), $this->environment->getNrOrgTrab(), $this->environment->getCdOperador());
                        }
                    }
                }
                $this->geraReplicacaoAutomatica = !!$replicacoes;
            }
            
            if($this->replicacaoManual || $documentos || $this->geraReplicacaoAutomatica){
                $nrregistro = $nrregistro === 0 ? 1 : $nrregistro;
                $proxCodigoNota = $this->gerarNovosCodigos($nrregistro, self::NOVO_CODIGO_NOTAFISCAL.$this->environment->getCdFilial(), $this->environment->getNrOrgTrab());
                $proxCodigoEstoque = $this->gerarNovosCodigos($nrregistro, self::NOVO_CODIGO_ESTOQUE.$this->environment->getCdFilial(), $this->environment->getNrOrgTrab());
            
                $notasOrigemReplicacao = $this->replicacaoDocumentosFactory->getNotasOrigemReplicacao($this->notafiscal->getCdfilial() . $this->notafiscal->getNrlanctonf());
                foreach ($notasOrigemReplicacao as $notaOrigemRep) {
                    $this->proceduresNfFactory->deletaNf($notaOrigemRep['CDFILIAL'], $notaOrigemRep['NRLANCTONF'], 'I');
                    $this->proceduresNfFactory->deletaNf($notaOrigemRep['CDFILIAL'], $notaOrigemRep['NRLANCTONF'], 'C');
                    
                    $filialMovimentacao = $notaOrigemRep['IDENTRSAIDOP'] === 'E' ? $notaOrigemRep['CDFILIRECE'] : $notaOrigemRep['CDFILIEMIT'];
                    $paramFilialMovimentacao = $this->buscarParamfilial($filialMovimentacao);
                    
                    if($paramFilialMovimentacao->getIdestonline() !== 'N')
                        $this->verificarEstoque($notaOrigemRep);
                }
                
                $notafiscal = $this->replicacaoDocumentosFactory->getNotafiscal($this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf());
                if($this->notasDeletadas){
                    $this->proceduresNfFactory->replicacaoDocumentos($notafiscal['CDFILIAL'], $notafiscal['NRLANCTONF'], $this->environment->getCdFilial(),
                                                    $proxCodigoNota, $proxCodigoEstoque, $notafiscal['IDDOCFATUR']);
                    $this->verificarReplicacao($notafiscal);
                    $this->verificarOrcamento($notafiscal);
                }
                $this->replicacaoDocumentosFactory->deletarTodaReplicacaoTemporaria();
            }
            if($this->arrayErrosReplicacao){
                $connection->rollback();
                return $this->arrayErrosReplicacao;
            }
            
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollback();
            $this->replicacaoDocumentosFactory->deletarTodaReplicacaoTemporaria();
            throw $e;
        }
    }
    
    private function gerarNovosCodigos($numRegistros, $contador, $organizacao){
        for($i = 0; $i <= $numRegistros; $i++) {
            $proxCodigo[] = $this->novoCodigo->getCodeByTableName($contador, $organizacao);
        }
        
        return $proxCodigo[0];
    }
    
    private function verificarOrcamento($notafiscal){
        $replicacoes = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Notafiscal')->findBy(
                                                                    array('iddocreplic' => $notafiscal['CDFILIAL'] . $notafiscal['NRLANCTONF']));
        foreach ($replicacoes as $replic) {
            if($replic->getIdentrsaidop() === 'E'){
                $this->liberarOrcamento($replic->getCdfilirece(), $replic->getCdfilial(), 
                    $replic->getNrlanctonf(), $replic->getCdtipooper(), $replic->getDtentrsaid());
            }
        }
    }
    
    private function liberarOrcamento($filirece, $filial, $lancamento, $operacao, $dataLancamento, $filialOrigem = '', $lancamentoOrigem = ''){
        $filialOrcamentoGOP = $this->replicacaoDocumentosFactory->verificarFilialUtilizaOrcamentoGOP($filirece)[0];
        if($filialOrcamentoGOP['IDCONSORCFIL'] !== 'S')
            return false;
            
        $vrAlertOrc = 0;
        $ano = DateUtil::getAno($dataLancamento);
        $mes = DateUtil::getMes($dataLancamento);
        $versao = $this->replicacaoDocumentosFactory->buscarMaiorVersaoOrcamento($ano);
        
        $this->verificarOrcamentoConsumo($filialOrcamentoGOP, $filial, $lancamento, $filirece, $dataLancamento, $operacao, $ano, $mes, $versao);
        $this->verificarOrcamentoDespesa($filialOrcamentoGOP, $filial, $lancamento, $filirece, $operacao, $ano, $mes, $versao);
    }
    
    private function verificarOrcamentoConsumo($filialOrcamentoGOP, $filial, $lancamento, $filirece, $dataLancamento, $operacao, $ano, $mes, $versao){
        $orcamentoConsumo = $this->replicacaoDocumentosFactory->buscarOrcamentoConsumo($filirece, $ano, $mes, $versao);
        
        if(empty($orcamentoConsumo))
            return false;
            
        $consumo = $this->replicacaoDocumentosFactory->buscarConsumo($filial, $lancamento, $filirece, 'S');
        
        if($consumo['IDCNTRESTOQ'] !== 'S')
            return false;
            
        $operacao = $this->tipooperac = $this->entityManager->getRepository('Teknisa\Model\Geral\Entities\Tipooperac')->findOneBy(
            array('identrsaidop' => 'E',
                  'cdtipooper' => $operacao));
                  
        if($operacao->getIdatuesttpop() !== 'S')
            return false;
            
        $data = DateUtil::getDataDeString($dataLancamento);
        $entradas = $this->replicacaoDocumentosFactory->verificarEntradas($filirece, DateUtil::getPrimeiroDiaDoMes($data), DateUtil::getUltimoDiaDoMes($data));
        
        if($entradas['VRTOTALENTR'] === 0 || empty($entradas['VRTOTALENTR']))
            return false;
            
        $orcamento = $filialOrcamentoGOP['VRPEORCBLOQFIL'] != 0 && !empty($filialOrcamentoGOP['VRPEORCBLOQFIL']) ? 
                            $orcamentoConsumo['VRORCAMENTOC'] + (($orcamentoConsumo['VRORCAMENTOC'] * $filialOrcamentoGOP['VRPEORCBLOQFIL']) / 100) :
                            $orcamentoConsumo['VRORCAMENTOC'];
                            
        if($entradas['VRTOTALENTR'] > $orcamento){
            $this->arrayErrosReplicacao[] = array("DSERRO" => "Operação bloqueada! Lançamento de entrada para filial {$filirece} excede valor de consumo orçado para o mês {$mes}.");
            return false;
        }
        
        if($filialOrcamentoGOP['VRPEORCALERTFIL'] != 0 && !empty($filialOrcamentoGOP['VRPEORCALERTFIL'])){
            $alertaOrcamento = $this->alertaOrcamento('C', $filialOrcamentoGOP, $entradas, $orcamentoConsumo);
            if($alertaOrcamento >= $filialOrcamentoGOP['VRPEORCALERTFIL']){
                $this->arrayErrosReplicacao[] = array("DSERRO" => "Atenção! Filial {$filirece} atingiu o valor orçado para o consumo do mês {$mes}");
                return false;
            }
        }
    }
    
    private function verificarOrcamentoDespesa($filialOrcamentoGOP, $filial, $lancamento, $filirece, $operacao, $ano, $mes, $versao){
        $orcamentoDespesa = $this->replicacaoDocumentosFactory->buscarOrcamentoDespesa($filirece, $ano, $mes, $versao, $operacao);
        
        if(empty($orcamentoDespesa))
            return false;
            
        $consumo = $this->replicacaoDocumentosFactory->buscarConsumo($filial, $lancamento, $filirece, 'S');
        
        if($consumo['IDCNTRESTOQ'] !== 'N')
            return false;
            
        $despesas = $this->replicacaoDocumentosFactory->verificarDespesas($filirece, DateUtil::getPrimeiroDiaDoMes($data), DateUtil::getUltimoDiaDoMes($data), $operacao);
        
        if($despesas['VRDESPESA'] === 0 || empty($despesas['VRDESPESA']))
            return false;
            
        $orcamento = $filialOrcamentoGOP['VRPEORCBLOQFIL'] != 0 && !empty($filialOrcamentoGOP['VRPEORCBLOQFIL']) ? 
                            $orcamentoDespesa['VRORCAMENTOE'] + (($orcamentoDespesa['VRORCAMENTOE'] * $filialOrcamentoGOP['VRPEORCBLOQFIL']) / 100) :
                            $orcamentoDespesa['VRORCAMENTOE'];
                            
        if($despesas['VRDESPESA'] > $orcamento){
            $this->arrayErrosReplicacao[] = array("DSERRO" => "Operação bloqueada! Lançamento de entrada para filial {$filirece} excede valor de despesa orçado para o mês {$mes}.");
            return false;
        }
        
        if($filialOrcamentoGOP['VRPEORCALERTFIL'] != 0 && !empty($filialOrcamentoGOP['VRPEORCALERTFIL'])){
            $alertaOrcamento = $this->alertaOrcamento('E', $filialOrcamentoGOP, $despesas, $orcamentoDespesa);
            if($alertaOrcamento >= $filialOrcamentoGOP['VRPEORCALERTFIL']){
                $this->arrayErrosReplicacao[] = array("DSERRO" => "Atenção! Filial {$filirece} atingiu o valor orçado para o consumo do mês {$mes}");
                return false;
            }
        }
    }
    
    private function alertaOrcamento($tipoOperacao, $filialOrcamentoGOP, $valor, $valorOrcamento){
        $porcentAlert = 0;
        
        if($filialOrcamentoGOP['VRPEORCALERTFIL'] !== 0 && !empty($filialOrcamentoGOP['VRPEORCALERTFIL'])){
            switch ($tipoOperacao) {
                case 'C':
                    $porcentAlert = ($valor['VRTOTALENTR'] * 100) / $valorOrcamento['VRORCAMENTOC'];
                    break;
                case 'E':
                    $porcentAlert = ($valor['VRDESPESA'] * 100) / $valorOrcamento['VRORCAMENTOE'];
                    break;
                // case 'O':
                //     $porcentAlert = ($valor['TOTDESPMO'] * 100) / $valorOrcamento['VRORCAMENTO'];
                //     break;
                // case 'D':
                //     $porcentAlert = ($valor['TOTDESPDIV'] * 100) / $valorOrcamento['VRORCAMENTO'];
                //     break;
            }
        }
        
        return round($porcentAlert, 2);
    }
    
    private function verificarReplicacao($notafiscal){
        $replicacao = $this->replicacaoDocumentosFactory->verificarReplicacao($notafiscal['CDFILIAL'].$notafiscal['NRLANCTONF']);
        foreach($replicacao as $replic){
            $filialMovimentacao = $replic['IDENTRSAIDOP'] === 'E' ? $replic['CDFILIRECE'] : $replic['CDFILIEMIT'];
            $paramFilialMovimentacao = $this->buscarParamfilial($filialMovimentacao);
            
            $this->verificarFilialOrigem($replic, $filialMovimentacao);
            $this->atualizarEstoque($replic['CDFILIAL'], $replic['NRLANCESTQ']);
        }
    }
    
    private function atualizarEstoque($filial, $nrlancestq){
        try {
            $this->validacoesEstoqueService->atualizaEstoque($filial, $nrlancestq);
        } catch (\Exception $e) {
            $this->arrayErrosReplicacao[] = array("DSERRO" => $e->getMessage());
        }
    }
    
    private function verificarEstoque($nota){
        try {
            $lanctoestoq = Lanctoestoq::findOneByCdfilialAndNrlancestqAndNrorg($nota['CDFILIAL'], $nota['NRLANCESTQ'], $nota['NRORG']);
            if ($lanctoestoq)
                $this->validacoesEstoqueService->verificaIrregularidades($lanctoestoq);
        } catch (\Exception $e) {
            $this->arrayErrosReplicacao[] = array("DSERRO" => $e->getMessage());
            $this->notasDeletadas = false;
        }
    }
    
    private function buscarParamfilial($filial){
        return $this->entityManager->getRepository('Teknisa\Model\Geral\Entities\Paramfilial')->findOneBy(array('cdfilial' => $filial));
    }
    
    private function verificarUtilizacaoNumeracaoTransferencia(){
        $this->tipooperac = $this->entityManager->getRepository('Teknisa\Model\Geral\Entities\Tipooperac')->findOneBy(
            array('identrsaidop' => $this->notafiscal->getIdentrsaidop(),
                  'cdtipooper' => $this->notafiscal->getCdtipooper()));

        if($this->tipooperac->getIdutgernftran() === 'S'){
            $this->arrayErrosReplicacao[] = array("DSERRO" => "Operação Bloqueada! A replicação de documentos da importação não pode ser concluída pois o tipo de operação
               de saída {$this->tipooperac->getNmtipooper()} utiliza geração de NF de transferência.");
               
            return false;
        }
        
        return true;
    }
    
    private function verificarFilialOrigem($replicacao, $filialMovimentacao){
        $replicacaoEstoque = $this->replicacaoDocumentosFactory->verificarReplicacaoEstoque($replicacao['CDFILIAL'], $replicacao['NRLANCTONF'], $filialMovimentacao);
        $almoxarifado = false;
        $localestoque = false;
        $lote = false;
        
        foreach ($replicacaoEstoque as $replicEstoque) {
            if(!$almoxarifado)
                $almoxarifado = $this->validarFilialUtilizaAlmoxarifado($replicEstoque, $filialMovimentacao);
            if(!$localestoque)
                $localestoque = $this->validarFilialUtilizaLocalEstoque($replicEstoque, $filialMovimentacao);
            if(!$lote)
                $lote = $this->validarFilialUtilizaLote($replicEstoque, $filialMovimentacao);
        }
    }
    
    private function validarFilialUtilizaAlmoxarifado($replicEstoque, $filialMovimentacao){
        if($replicEstoque['IDUTILALMOX'] === 'S' && empty($replicEstoque['CDALMOXARIFE']) && $replicEstoque['IDCNTRESTOQ'] === 'S'){
            $this->arrayErrosReplicacao[] = array("DSERRO" => "Operação Bloqueada! Nota replicada contém filial {$filialMovimentacao} que utiliza controle por almoxarifado mas não há valor parametrizado.");
            return true;
        }
    }
    
    private function validarFilialUtilizaLocalEstoque($replicEstoque, $filialMovimentacao){
        if($replicEstoque['IDUTILLCESTQ'] === 'S' && empty($replicEstoque['CDLOCALESTOQ']) && $replicEstoque['IDCNTRESTOQ'] === 'S'){
            $this->arrayErrosReplicacao[] = array("DSERRO" => "Operação Bloqueada! Nota replicada contém filial {$filialMovimentacao} que utiliza controle por local de estoque mas não há valor parametrizado.");
            return true;
        }
    }
    
    private function validarFilialUtilizaLote($replicEstoque, $filialMovimentacao){
        if($replicEstoque['IDUTILLOTE'] === 'S' && empty($replicEstoque['NRLOTEESTQ']) && $replicEstoque['IDCNTRESTOQ'] === 'S'){
            $this->arrayErrosReplicacao[] = array("DSERRO" => "Operação Bloqueada! Nota replicada contém filial {$filialMovimentacao} que utiliza controle por lote mas não há valor parametrizado.");
            return true;
        }
    }
    
    private function verificarFilialDestino(){
        $filial = $this->entityManager->getRepository('Teknisa\Model\Geral\Entities\Filial')->findOneBy(array(
            'cdfilial' => $this->notafiscal->getCdfilirece()));
        
        $this->validarFilialInativa($filial);
        $this->validarFilialOperadorLogistico($filial);
        $this->validarFilialProdutoInativo($filial);
        $this->validarFilialProdutoSegmento();
    }
    
    private function validarFilialInativa($filial){
        if($filial->getIdcntrresc() === 'S'){
            $this->arrayErrosReplicacao[] = array("DSERRO" => "Operação Bloqueada! Filial de Destino {$filial->getNmfilial()} encontra-se inativa.");
        }
    }
    
    private function validarFilialOperadorLogistico($filial){
        if($filial->getIdfuncaofili() === 'O'){
            $this->paramgeral = $this->entityManager->getRepository('Teknisa\Model\Geral\Entities\Paramgeral')->find();
            if($this->paramgeral->getIdperreplicnfol() !== 'S'){
                $this->arrayErrosReplicacao[] = array("DSERRO" => "Operação Bloqueada! A replicação de documentos não pode ser concluída pois a filial de destino {$filial->getNmfilial()} é um operador logístico.");
            }
        }
    }
    
    private function validarFilialProdutoInativo($filial){
        $produtoInativo = $this->replicacaoDocumentosFactory->produtoInativo($this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf(), $filial->getCdfilial());
        if($produtoInativo){
            foreach ($produtoInativo as $produto) {
                $this->arrayErrosReplicacao[] = array("DSERRO" => "Produto {$produto['CDARVPROD']} inativo para a filial de destino {$filial}");
            }
        }
    }
    
    private function validarFilialProdutoSegmento(){
        $produtoSegmento = $this->replicacaoDocumentosFactory->produtoSegmento($this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf());
        if($produtoSegmento){
            foreach ($produtoSegmento as $produto) {
                $stRetSegment = $this->verificarSegmento($produto['FILIAL'], $produto['SEGMENTO'], $produto['MODPRODFIL'], $produto['PRODUTO']);
                if($stRetSegment){
                    $this->arrayErrosReplicacao[] = array("DSERRO" => $stRetSegment);
                }
            }
        }
    }
    
    private function verificarSegmento($filial, $segmento, $utilizaModuloFilial, $produto){
        if($segmento){
            if(empty($utilizaModuloFilial) || $utilizaModuloFilial === 'N'){
                return $this->verificarProdutoSegmento($produto, $segmento, $filial);
            }
            else{
                return $this->verificarProdutoSegmentoModuloFilial($produto, $segmento, $filial);
            }
        }
    }
    
    private function verificarProdutoSegmento($produto, $segmento, $filial){
        $ProdutoSegmento = $this->replicacaoDocumentosFactory->getProdutoSegmento($produto, $segmento);
        if(empty($ProdutoSegmento)){
            return "Produto(s) {$produto['NMPRODUTO']} não cadastrado(s) para o segmento de mercado da filial {$filial}.";
        }
        else if($ProdutoSegmento['IDPROSEGATIV'] === 'N'){
            return "Produto(s) {$produto['NMPRODUTO']} inativo(s) para o segmento de mercado da filial {$filial}.";
        }
    }
    
    private function verificarProdutoSegmentoModuloFilial($produto, $segmento, $filial){
        $ProdutoSegmentoModuloFilial = $this->replicacaoDocumentosFactory->getProdutoSegmentoModuloFilial($produto, $segmento, $filial);
        if(empty($ProdutoSegmentoModuloFilial)){
            return "Produto {$produto['NMPRODUTO']} não cadastrado para o segmento de mercado e/ou módulo da filial {$filial}.";
        }
        else if($ProdutoSegmentoModuloFilial['IDPROSEGATIV'] === 'N'){
            return "Produto(s) {$produto['NMPRODUTO']} inativo(s) para o segmento de mercado da filial {$filial}.";
        }
    }
    
    private function verificarDuplicidadeDeNotas($filial, $lancamento, $filialOrigem, $filialDestino, $entradaSaida, $operacao, $almoxarifado, $localEstoque){
        $itensTabelaTemporaria = $this->replicacaoDocumentosFactory->getDocumentosTabelaTemporaria($filial, $lancamento, $filialOrigem, 
                                            $filialDestino, $entradaSaida, $operacao, $almoxarifado, $localEstoque);
        if($itensTabelaTemporaria){
            $this->arrayErrosReplicacao[] = array("DSERRO" => "Operação bloqueada. A replicação de documentos está gerando duplicidade de notas fiscais. 
                Favor verificar o cadastro na tela de Replicação de Documentos.");
                
            return false;
        }
        
        return true;
    }
    
    private function verificarData($data, $filialMovimentacao){
        $result = true;
        $parametrosDFS = $this->replicacaoDocumentosFactory->parametrosDFS($filialMovimentacao);
        if($parametrosDFS){
            if($data instanceof DateTime === false)
                $data = DateUtil::getDataDeString($data, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            if($this->dataProcessamento instanceof DateTime === false)
                $dataProcessamento = DateUtil::getDataDeString($this->dataProcessamento, DateUtil::FORMATO_BRASILEIRO_DATAHORA);                
                
            if($parametrosDFS['IDLANCDTMAI'] === 'N' && $parametrosDFS['IDLANCDTMAIFIL'] === 'N' && DateUtil::comparaDatas($dataProcessamento, $data)){
                $this->arrayErrosReplicacao[] = array("DSERRO" => "Data informada não deve ser maior que a data de processamento.");
                $result = false;
            }
            
            if($parametrosDFS['IDLANCDTMEN'] === 'N' && $parametrosDFS['IDLANCDTMENFIL'] === 'N' && DateUtil::comparaDatas($data, $dataProcessamento)){
                $this->arrayErrosReplicacao[] = array("DSERRO" => "Data informada não deve ser menor que a data de processamento.");
                $result = false;
            }
            
            if($parametrosDFS['IDLANCDTMENFIL'] === 'S' && DateUtil::comparaDatas($data, $dataProcessamento)){
                if(DateUtil::comparaDatas($data, DateUtil::subtraiIntervalo($dataProcessamento, $parametrosDFS['NRDIASSAIDFIL']))){
                    $this->arrayErrosReplicacao[] = array("DSERRO" => "Data informada fora do período permitido.");
                    $result = false;
                }
            }
            else if($parametrosDFS['IDLANCDTMEN'] === 'S' && DateUtil::comparaDatas($data, $dataProcessamento)){
                if(DateUtil::comparaDatas($data, DateUtil::subtraiIntervalo($dataProcessamento, $parametrosDFS['NRDIASSAID']))){
                    $this->arrayErrosReplicacao[] = array("DSERRO" => "Data informada fora do período permitido.");
                    $result = false;
                }
            }
            
            if($parametrosDFS['IDLANCDTMAIFIL'] === 'S' && DateUtil::comparaDatas($dataProcessamento, $data)){
                if(DateUtil::comparaDatas(DateUtil::adicionaIntervalo($dataProcessamento, $parametrosDFS['NRDIASSAIDPDFIL']), $data)){
                    $this->arrayErrosReplicacao[] = array("DSERRO" => "Data informada fora do período permitido.");
                    $result = false;
                }
            }
            else if($parametrosDFS['IDLANCDTMAI'] === 'S' && DateUtil::comparaDatas($dataProcessamento, $data)){
                if(DateUtil::comparaDatas(DateUtil::adicionaIntervalo($dataProcessamento, $parametrosDFS['NRDIASSAIDPD']), $data)){
                    $this->arrayErrosReplicacao[] = array("DSERRO" => "Data informada fora do período permitido.");
                    $result = false;
                }
            }
        }
        
        return $result;
    }
    
    private function verificarExisteOperacaoEntrada(){
        $operacaoEntrada = $this->entityManager->getRepository('Teknisa\Model\Geral\Entities\Tipooperac')->findOneBy(
            array('identrsaidop' => 'E', 'cdtipooper' => $this->notafiscal->getCdtipooper()));
        if(empty($operacaoEntrada))
            $this->arrayErrosReplicacao[] = array("DSERRO" => "Operação Bloqueada! A replicação de documentos da importação não pode 
                ser concluída pois o tipo de operação {$this->notafiscal->getCdtipooper()} não existe na entrada.");
        return $operacaoEntrada;
    }
    
    private function verificarExisteSerieEntrada(){
        $serieEntrada = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Serieentr')->findOneBy(
            array('cdserieentr' => $this->notafiscal->getIdentrsaidop() == 'E'? $this->notafiscal->getCdserieentr() : $this->notafiscal->getCdserie()));
        if(empty($serieEntrada))
            $this->arrayErrosReplicacao[] = array("DSERRO" => "Operação Bloqueada! A replicação de documentos da importação 
                não pode ser concluída pois a série {$this->notafiscal->getCdserieentr()} não existe na entrada.");
        return $serieEntrada;
    }
}